#!/bin/bash
#
# buildCaffe.sh 

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e14|e15|e17> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=caffe
origpkgver=v1_0
pkgver=${origpkgver}k
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
ssibuildshims_version=v1_04_06
pkgtarname=caffe-${pkgdotver}

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir} ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [[ "${basequal}" == e1[045] ]]
then
  cxxflg="-std=c++14"
elif [[ "${basequal}" == e17 ]]
then
  cxxflg="-std=c++17"
else
   echo "ERROR: unrecognized qualifier ${basequal}"
   usage
   exit 1
fi

if [ "${extraqual}" = "debug" ]
then
    extra_command="-DCMAKE_BUILD_TYPE=Debug"
elif [ "${extraqual}" = "prof" ]
then
    extra_command="-DCMAKE_BUILD_TYPE=MinSizeRel"
else
   echo "ERROR: please specify debug or prof"
   usage
   exit 1
fi

rm -rf ${pkgdir}
mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
rm -rf ${blddir}
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
rm -rf ${sourcedir}
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake || ssi_die "ERROR: failed to setup camke"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

# double check that dependencies have been setup
echo
if [ -z ${LMDB_INC} ]
then
   echo "ERROR: failed to setup lmdb"
   exit 1
else
   echo "LMDB_INC: ${LMDB_INC}"
fi
if [ -z ${PROTOBUF_INC} ]
then
   echo "ERROR: failed to setup protobuf"
   exit 1
else
   echo "PROTOBUF_INC: ${PROTOBUF_INC}"
fi
if [ -z ${GLOG_INC} ]
then
   echo "ERROR: failed to setup glog"
   exit 1
else
   echo "GLOG_INC: ${GLOG_INC}"
fi
if [ -z ${GFLAGS_INC} ]
then
   echo "ERROR: failed to setup gflags"
   exit 1
else
   echo "GFLAGS_INC: ${GFLAGS_INC}"
fi
if [ -z ${LEVELDB_INC} ]
then
   echo "ERROR: failed to setup leveldb"
   exit 1
else
   echo "LEVELDB_INC: ${LEVELDB_INC}"
fi
if [ -z ${SNAPPY_INC} ]
then
   echo "ERROR: failed to setup snappy"
   exit 1
else
   echo "SNAPPY_INC: ${SNAPPY_INC}"
fi
if [ -z ${GSL_INC} ]
then
   echo "ERROR: failed to setup gsl"
   exit 1
else
   echo "GSL_INC: ${GSL_INC}"
fi
if [ -z ${BOOST_INC} ]
then
   echo "ERROR: failed to setup boost"
   exit 1
else
   echo "BOOST_INC: ${BOOST_INC}"
fi
if [ -z ${LAPACK_LIB} ]
then
   echo "ERROR: failed to setup lapack"
   exit 1
else
   echo "LAPACK_LIB: ${LAPACK_LIB}"
fi
if [ -z ${HDF5_INC} ]
then
   echo "ERROR: failed to setup hdf5"
   exit 1
else
   echo "HDF5_INC: ${HDF5_INC}"
fi
if [ -z ${OPENCV_INC} ]
then
   echo "ERROR: failed to setup opencv"
   exit 1
else
   echo "OPENCV_INC: ${OPENCV_INC}"
fi
echo

cd ${sourcedir}
tar xf ${tardir}/${pkgtarname}.tar.gz

# apply Fermi patches
for patch in ${patchdir}/${pkgtarname}.patch; do
  patch ${DEBUG_PATCHES:+--verbose} -p0 < "${patch}" || ssi_die "ERROR: application of patch ${patch} failed."
done

set -x

cd ${blddir}

##env CC=gcc CXX=g++ FC=gfortran cmake -DCMAKE_INSTALL_PREFIX=${pkgdir} -DGLOG_ROOT_DIR=$GLOG_FQ_DIR ../caffe-1.0
##    -DAtlas_ROOT_DIR=${LAPACK_FQ_DIR} \

env CC=gcc CXX=g++ FC=gfortran cmake \
    -DCMAKE_INSTALL_PREFIX=${pkgdir} \
    -DCPU_ONLY=ON \
    -DCMAKE_CXX_FLAGS="${cxxflg}" \
    ${extra_command} \
    -DBUILD_python=OFF \
    -DAtlas_ROOT_DIR=${LAPACK_FQ_DIR} \
    ${sourcedir}/${pkgtarname}

make -j${ncore} || ssi_die "make failed."
make test || ssi_die "make test failed."
make runtest || ssi_die "make runtest failed."
make -j${ncore} install || ssi_die "make install failed."

set +x

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || ssi_die "failed to declare ${package} ${pkgver}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "caffe is installed at ${CAFFE_FQ_DIR}"

# fix paths for cmake config
cmakeConfig=`ls ${CAFFE_FQ_DIR}/share/Caffe/*.cmake`
for file in ${cmakeConfig}
do
  mv ${file} ${file}.bak
  cat ${file}.bak | sed -e "s%${CAFFE_FQ_DIR}%\$ENV{CAFFE_FQ_DIR}%g" | \
                    sed -e "s%${OPENCV_FQ_DIR}%\$ENV{OPENCV_FQ_DIR}%g" | \
                    sed -e "s%${BOOST_FQ_DIR}%\$ENV{BOOST_FQ_DIR}%g" | \
                    sed -e "s%${GLOG_FQ_DIR}%\$ENV{GLOG_FQ_DIR}%g" | \
                    sed -e "s%${GFLAGS_FQ_DIR}%\$ENV{GFLAGS_FQ_DIR}%g" | \
                    sed -e "s%${PROTOBUF_FQ_DIR}%\$ENV{PROTOBUF_FQ_DIR}%g" | \
                    sed -e "s%${HDF5_FQ_DIR}%\$ENV{HDF5_FQ_DIR}%g" | \
                    sed -e "s%${LMDB_FQ_DIR}%\$ENV{LMDB_FQ_DIR}%g" | \
                    sed -e "s%${LEVELDB_FQ_DIR}%\$ENV{LEVELDB_FQ_DIR}%g" | \
                    sed -e "s%${LAPACK_FQ_DIR}%\$ENV{LAPACK_FQ_DIR}%g" | \
		    sed -e "s%${GCC_FQ_DIR}%\$ENV{GCC_FQ_DIR}%g" > ${file}
done


# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${CAFFE_FQ_DIR} ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
